# -*- coding:utf-8 -*-

import sys
import csv
import urllib.request
import bs4
import bs4.element
import re

recompiled = re.compile(r"[（(].+[)）]")

def getdata(code_number):
    """
    :param code_number: 証券コード。文字列。
    :type code_number: str
    :rtype: dict
    :return: Yahoo!ファイナンスから取得した各種データ
    """
    url  = "http://stocks.finance.yahoo.co.jp/stocks/detail/?code=" + code_number
    req  = urllib.request.urlopen(url)
    html = req.read().decode()
    bs   = bs4.BeautifulSoup(html)
    dic  = {"証券コード" : code_number}

    if bs.select(".selectFinTitle.yjL"):
        allert = bs.select(".selectFinTitle.yjL")[0].text
        print("証券コード:{0:<5}の{1}".format(code_number, allert))
        dic["社名"] = allert
    else:
        dic["市場"]   = bs.select(".stockMainTabName")[0].text
        dic["社名"] = bs.select("h1")[0].text
        print("証券コード: {0:<8}社名: {1}  を読み込んでいます...".format(
            dic["証券コード"], dic["社名"]))
        for tag in bs.select(".lineFi.clearfix dl"):
            assert isinstance(tag, bs4.element.Tag)
            # 辞書のキー名を取得
            name  = tag.select("dt")[0].text.strip()
            name  = name.split("\n")[0]
            # 辞書の値を取得
            value = tag.select("dd strong")[0].text
            value = value.strip()
            value = recompiled.sub("", value)
            dic[name] = value
    return dic


if __name__ == '__main__':
    path = ""
    try:
        path = sys.argv[1]
    except:
        input("ファイルをドラッグ＆ドロップしてください")
        sys.exit(1)

    print('CSVファイル "{0}" を読み込みます\n'.format(path))
    f = open(path, mode="r", encoding="cp932", newline="")
    csvr = csv.DictReader(f)

    filedname = (
        "証券コード", "市場", "社名", "前日終値", "始値", "高値", "安値", "出来高",
        "売買代金", "値幅制限", "時価総額", "発行済株式数", "配当利回り（会社予想）",
        "1株配当（会社予想）", "PER（会社予想）", "PBR（実績）", "EPS（会社予想）",
        "BPS（実績）", "最低購入代金", "単元株数", "年初来高値", "年初来安値", "信用買残",
        "前週比", "信用売残", "前週比"
    )
    w = open("株価のデータ.csv", mode="w", encoding="cp932", newline="")
    w.write(",".join(filedname)+"\n")
    csvw = csv.DictWriter(w, filedname)


    for line in csvr:
        dic = getdata(line["証券コード"])
        csvw.writerow(dic)

    f.close()
    w.close()
    input("データの取り込みが完了しました。Enterキーを押してください")
