# -*- coding:utf-8 -*-

import sys
import csv
import urllib.request
import bs4
import bs4.element
import re

recompiled = re.compile(r"[（(].+[)）]")

def getdata(code_number):
    """
    :param code_number: 証券コード。文字列。
    :type code_number: str
    :rtype: dict
    :return: Yahoo!ファイナンスから取得した各種データ
    """
    url  = "http://profile.yahoo.co.jp/consolidate/" + code_number
    req  = urllib.request.urlopen(url)
    html = req.read().decode()
    bs   = bs4.BeautifulSoup(html)
    dic  = {"証券コード" : code_number}

    if bs.select(".selectFinTitle.yjL"):
        allert = bs.select(".selectFinTitle")[0].text
        print("証券コード:{0:<5}の{1}".format(code_number, allert))
        dic["社名"] = allert
    else:
        dic["社名"] = bs.select("h1")[0].text.strip()
        print("証券コード: {0:<8}社名: {1}  を読み込んでいます...".format(
            dic["証券コード"], dic["社名"]))
        for i, tag in enumerate(bs.select(".yjMt tr")):
            if i == 0:
                continue
            title = tag.select("td")[0].text
            value = tag.select("td")[1].text
            value = value.replace("百万", "000000")
            value = value.replace("円", "")
            dic[title] = value
    return dic


if __name__ == '__main__':
    path = ""
    try:
        path = sys.argv[1]
    except:
        input("ファイルをドラッグ＆ドロップしてください")
        sys.exit(1)

    print('CSVファイル "{0}" を読み込みます\n'.format(path))
    f = open(path, mode="r", encoding="cp932", newline="")
    csvr = csv.DictReader(f)

    filedname = (
        "証券コード", "社名", "決算期", "会計方式", "決算発表日", "決算月数",
        "売上高", "営業利益", "経常利益", "当期利益", "EPS（一株当たり利益）",
        "調整一株当たり利益", "BPS（一株当たり純資産）", "総資産", "自己資本",
        "資本金", "有利子負債", "自己資本比率", "ROA（総資産利益率）",
        "ROE（自己資本利益率）", "総資産経常利益率"
    )
    w = open("連結決算のデータ.csv", mode="w", encoding="cp932", newline="")
    w.write(",".join(filedname)+"\n")
    csvw = csv.DictWriter(w, filedname)


    for line in csvr:
        dic = getdata(line["証券コード"])
        csvw.writerow(dic)

    f.close()
    w.close()
    input("データの取り込みが完了しました。Enterキーを押してください")
